@@ -5,7 +5,6 @@ import os
 import re
 import sys
 from dataclasses import dataclass, field
-from http.cookies import Morsel  # noqa
 from pathlib import Path
 from types import SimpleNamespace
 from typing import (
@@ -24,6 +23,8 @@ import aiohttp
 import click
 from rich.console import Console, PagerContext
 from rich.pager import Pager
+from rich.status import Status
+from rich.text import Text as RichText
 
 from neuro_sdk import Client, ConfigError, Factory, gen_trace_id
 from neuro_sdk.config import _ConfigData, load_user_config
@@ -137,6 +138,8 @@ class Root:
         finally:
             sys.stderr = sys.__stderr__
 
+        self.soft_reset_tty()
+
     def run(self, main: Awaitable[_T]) -> _T:
         return self._runner.run(main)
 
@@ -167,14 +170,9 @@ class Root:
     @property
     def factory(self) -> Factory:
         if self._factory is None:
-            trace_configs: Optional[List[aiohttp.TraceConfig]]
-            if self.trace:
-                trace_configs = [self._create_trace_config()]
-            else:
-                trace_configs = None
             self._factory = Factory(
                 path=self.config_path,
-                trace_configs=trace_configs,
+                trace_configs=[self._create_trace_config()],
                 trace_id=gen_trace_id(),
                 trace_sampled=True if self.force_trace_all else None,
             )
@@ -204,9 +202,11 @@ class Root:
         trace_config.on_response_chunk_received.append(self._on_response_chunk_received)
         return trace_config
 
-    def _print_debug(self, lines: List[str]) -> None:
+    def _print_trace(self, lines: List[str]) -> None:
         for line in lines:
-            self.print(line, style="dim", err=True)
+            if self.trace:
+                self.print(line, style="dim", err=True)
+            log.debug(line)
 
     def _process_chunk(self, chunk: bytes, printable: bool) -> List[str]:
         if not chunk:
@@ -231,7 +231,7 @@ class Root:
                 val = self._sanitize_header_value(val)
             lines.append(f"> {key}: {val}")
         lines.append("> ")
-        self._print_debug(lines)
+        self._print_trace(lines)
 
         content_type = data.headers.get("Content-Type", "")
         context.request_printable = content_type.startswith(TEXT_TYPE)
@@ -247,7 +247,7 @@ class Root:
             "> " + line
             for line in self._process_chunk(chunk, context.request_printable)
         ]
-        self._print_debug(lines)
+        self._print_trace(lines)
 
     async def _on_request_end(
         self,
@@ -258,7 +258,7 @@ class Root:
         lines = [f"< HTTP/1.1 {data.response.status} {data.response.reason}"]
         for key, val in data.response.headers.items():
             lines.append(f"< {key}: {val}")
-        self._print_debug(lines)
+        self._print_trace(lines)
 
         content_type = data.response.headers.get("Content-Type", "")
         context.response_printable = content_type.startswith(TEXT_TYPE)
@@ -274,7 +274,7 @@ class Root:
             "< " + line
             for line in self._process_chunk(chunk, context.response_printable)
         ]
-        self._print_debug(lines)
+        self._print_trace(lines)
 
     def _sanitize_header_value(self, text: str) -> str:
         for token in self._find_all_tokens(text):
@@ -315,6 +315,17 @@ class Root:
             sys.stdout.write("\x1b[!p")
             sys.stdout.flush()
 
+    @contextlib.contextmanager
+    def status(self, message: str) -> Iterator[Status]:
+        status = Status(
+            RichText.from_markup(message), console=self.console, spinner="dots"
+        )
+        if self.verbosity >= 0:
+            with status:
+                yield status
+        else:
+            yield status
+
     def pager(self) -> PagerContext:
         return self.console.pager(MaybePager(self.console), styles=True, links=True)
 
