@@ -30,11 +30,11 @@ def nmrc_path(tmp_path: Path, token: str, auth_config: _AuthConfig) -> Path:
     cluster_config = Cluster(
         registry_url=URL("https://registry-dev.neu.ro"),
         storage_url=URL("https://storage-dev.neu.ro"),
-        blob_storage_url=URL("https://blob-storage-dev.neu.ro"),
         users_url=URL("https://users-dev.neu.ro"),
         monitoring_url=URL("https://monitoring-dev.neu.ro"),
         secrets_url=URL("https://secrets-dev.neu.ro"),
         disks_url=URL("https://disks-dev.neu.ro"),
+        buckets_url=URL("https://buckets-dev.neu.ro"),
         presets={
             "gpu-small": Preset(
                 credits_per_hour=Decimal("10"),
@@ -62,11 +62,11 @@ def nmrc_path(tmp_path: Path, token: str, auth_config: _AuthConfig) -> Path:
     cluster2_config = Cluster(
         registry_url=URL("https://registry2-dev.neu.ro"),
         storage_url=URL("https://storage2-dev.neu.ro"),
-        blob_storage_url=URL("https://blob-storage2-dev.neu.ro"),
         users_url=URL("https://users2-dev.neu.ro"),
         monitoring_url=URL("https://monitoring2-dev.neu.ro"),
         secrets_url=URL("https://secrets2-dev.neu.ro"),
         disks_url=URL("https://disks2-dev.neu.ro"),
+        buckets_url=URL("https://buckets2-dev.neu.ro"),
         presets={
             "cpu-small": Preset(
                 credits_per_hour=Decimal("10"), cpu=7, memory_mb=2 * 1024
@@ -287,7 +287,8 @@ class RichComparator:
 
     def diff(self, lft: Guard, rgt: Guard) -> List[str]:
         # The same as _diff_text from
-        # https://github.com/pytest-dev/pytest/blob/master/src/_pytest/assertion/util.py#L200-L245 plus a few extra lines with additional instructions.  # noqa
+        # pytest/assertion/util.py#L200-L245
+        # plus a few extra lines with additional instructions.
         explanation: List[str] = []
 
         left = lft.arg
