@@ -55,6 +55,8 @@ def _job_entry(job_id: str, cluster_name: str = "default") -> Dict[str, Any]:
         "name": "my-job-name",
         "internal_hostname": f"job-id.{cluster_name}",
         "internal_hostname_named": f"my-job-name--job-owner.{cluster_name}",
+        "total_price_credits": "150",
+        "price_credits_per_hour": "15",
     }
 
 
@@ -475,9 +477,9 @@ def test_parse_file_resource_no_scheme(root: Root) -> None:
 
 
 def test_parse_file_resource_unsupported_scheme(root: Root) -> None:
-    with pytest.raises(ValueError, match=r"Unsupported URI scheme"):
+    with pytest.raises(ValueError, match=r"Invalid scheme"):
         parse_file_resource("http://neu.ro", root)
-    with pytest.raises(ValueError, match=r"Unsupported URI scheme"):
+    with pytest.raises(ValueError, match=r"Invalid scheme"):
         parse_file_resource("image:ubuntu", root)
 
 
@@ -554,11 +556,11 @@ def test_parse_resource_for_sharing_no_scheme(root: Root) -> None:
 
 
 def test_parse_resource_for_sharing_unsupported_scheme(root: Root) -> None:
-    with pytest.raises(ValueError, match=r"Unsupported URI scheme"):
+    with pytest.raises(ValueError, match=r"Invalid scheme"):
         parse_resource_for_sharing("http://neu.ro", root)
-    with pytest.raises(ValueError, match=r"Unsupported URI scheme"):
+    with pytest.raises(ValueError, match=r"Invalid scheme"):
         parse_resource_for_sharing("file:///etc/password", root)
-    with pytest.raises(ValueError, match=r"Unsupported URI scheme"):
+    with pytest.raises(ValueError, match=r"Invalid scheme"):
         parse_resource_for_sharing(r"c:scheme-less/resource", root)
 
 
